set echo on
clear screen

drop TABLE Address ;

CREATE TABLE Address (
  TRANS        NUMBER(4) PRIMARY KEY,
  EMPNO     NUMBER(4),
  ADDR_LINE1  VARCHAR2(40),
  ADDR_LINE2  VARCHAR2(40),
  CITY        VARCHAR2(40),
  STATE       VARCHAR2(2),
  ZIP         VARCHAR2(40),
  PREFERRED   VARCHAR2(1)
  );
pause 
clear screen


create or replace package state_pkg
as
    type array is table of number index by binary_integer;
    g_data array;
    procedure doit;
end;
/
pause 
clear screen


create or replace package body state_pkg
as

procedure doit
is
    l_idx  number;
    l_cnt1 number;
    l_cnt2 number;
begin
    l_idx := g_data.first;
    while (l_idx is not null)
    loop
        select count(*), count(decode(preferred,'Y','Y'))
          into l_cnt1, l_cnt2
          from Address
         where empno = l_idx;

        if ( l_cnt1 <> 0 and l_cnt2 <> 1 )
        then
            raise_application_error( -20001, 'At least/most one addr must be preferred' );
        end if;
        l_idx := g_data.next(l_idx);
    end loop;
end;

end;
/
pause
clear screen
 
create or replace trigger t1
before insert or update or delete on Address
begin
    state_pkg.g_data.delete;
end;
/
pause
clear screen
 

create or replace trigger t2
after insert or update or delete on Address for each row
begin
    if (inserting or updating)
    then
        state_pkg.g_data( :new.empno ) := 1;
    end if;
    if (deleting or updating)
    then
        state_pkg.g_data( :old.empno ) := 1;
    end if;
end;
/
pause
clear screen
 

create or replace trigger t3
after insert or update or delete on Address
begin
    state_pkg.doit;
end;
/
pause
clear screen
 
insert into Address values ( 2,   111, 'x', 'y', 'x', 'y', 'z', 'N' );
pause
clear screen

insert into Address values ( 1,   111, 'x', 'y', 'x', 'y', 'z', 'Y' );
insert into Address values ( 2,   111, 'x', 'y', 'x', 'y', 'z', 'N' );
insert into Address values ( 3,   111, 'x', 'y', 'x', 'y', 'z', 'N' );
insert into Address values ( 4,   111, 'x', 'y', 'x', 'y', 'z', 'Y' );
rollback;
pause
clear screen
 
 
insert into Address values ( 1,   111, 'x', 'y', 'x', 'y', 'z', 'Y' );
declare
    pragma autonomous_transaction;
begin
    insert into Address values ( 4,   111, 'x', 'y', 'x', 'y', 'z', 'Y' );
    commit;
end;
/
commit;
pause
clear screen

select empno, preferred from address;
pause
clear screen

delete from Address;
create UNIQUE index emp_addr_index on
Address( decode( preferred, 'Y', empno ) );
pause
clear screen


insert into Address values ( 1,   111, 'x', 'y', 'x', 'y', 'z', 'Y' );
declare
    pragma autonomous_transaction;
begin
    insert into Address values ( 4,   111, 'x', 'y', 'x', 'y', 'z', 'Y' );
    commit;
end;
/
commit;
